unit Principal_discretiza;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, discretiza, umodelosintcegh, matreal, xMatDefs, fddp;


 // Programa principal del discretizador que toma un archivo CEGH y un factor
 // de discretizacion y calcula las Matrices A y B del filtro asi como las
 // funciones deformantes del nuevo sintetizador

procedure Discretizar(paso_discr: integer; arch: string);

// Funcion que calcula la nueva matriz A del Sintetizador submuestreado

function TrasnformarA(MA: TMatR; n: integer): TMatR;

// Funcion que calcula la nueva matriz B del Sintetizador submuestreado

function TrasnformarB(MA, MB: TMatR; n: integer): TMatR;

 // Funcion que calcula la matriz P y D (diagonal) de una matriz A tal que
 // A=P*D*inv(P) donde D es una matriz diagonal formada por los autovalores de A
 // y P es la matriz cuyas columnas son los vectores propio asociados normalizados.
 // Esta funcion usa el metodo de las potencias y el metodo de deflaccion.
function Metodo_potencias(const MatA: TMatR; var VecProp: TMatR): TMatR;

// Funcion auxiliar para calcular un vector normalizado utilizando la norma infinito.

function Sobre_Max_val_abs(const Vector: TVectR; var Maximo: NReal): TVectR;

 // Funcion auxiliar para calcular los autovectores a partir de los autovalores
 // en el metodo de deflaccion.

function Proyeccion(landa1, landa2: NReal; v1, v2, x1: TVectR): TVectR;

 // Funcion para calcular la potencia real de una Matriz utilizando el
 // metodo de las potencias

function Matriz_Potencia_real(Matriz: TMatR; n: NReal): TMatR;

// Funcion auxiliar para calcular la potencia de un real  NO SABIA DONDE ESTABA

function Potencia(x, y: NReal): NReal;

// Funcion para interpolar las funciones deformantes cuando se discretiza

function Interpolacion_funciones_deformantes(alf, Inferior, Superior: NReal): NReal;

// Procedimiento auxiliar para mostrar los datos del sintetizador origen
procedure muestra(arch: string; var a, b, c: integer);



implementation

function Interpolacion_funciones_deformantes(alf, Inferior, Superior: NReal): NReal;
begin
  Result := Inferior; //alf*Inferior+(1-alf)*Superior+(Superior-Inferior)/2;
end;

function Potencia(x, y: NReal): NReal;
begin
  Potencia := Exp(y * Ln(x));
end;

function Matriz_Potencia_real(Matriz: TMatR; n: NReal): TMatR;
var
  temp, tempinv, autov, res: TMatR;
  x: Nreal;
  i: integer;
begin
  temp    := TMatR.Create_Init(Matriz.nf, Matriz.nc);
  tempinv := TMatR.Create_Init(Matriz.nf, Matriz.nc);
  autov   := TMatR.Create_Init(Matriz.nf, Matriz.nc);
  res     := TMatR.Create_Init(Matriz.nf, Matriz.nc);

  autov := Metodo_potencias(Matriz, temp);
  tempinv.Igual(temp);
  tempinv.Inv;
  x := 0;
  for i := 1 to Matriz.nc do
  begin
    x := autov.e(i, i);
    autov.pon_e(i, i, Potencia(x, n));
  end;


  temp.Mult(autov, res);
  temp.Igual(res);
  temp.Mult(tempinv, res);
  Result := res;

end;


function Proyeccion(landa1, landa2: NReal; v1, v2, x1: TVectR): TVectR;
var
  temp: TVectR;
begin
  temp := TVectR.Create_Init(v1.n);
  temp.Igual(v2);
  temp.PorReal(landa2 - landa1);
  temp.sumRPV(landa1 * x1.PEV(v2), v1);
  Result := temp;
end;


function Sobre_Max_val_abs(const Vector: TVectR; var Maximo: NReal): TVectR;
var
  Min, Max:   Nreal;
  kMin, kMax: integer;
  Vec_aux:    TVectR;

begin
  Vec_aux := TVectR.Create_Init(Vector.n);
  Vector.MinMax(kMin, kMax, Min, Max);
  Vec_aux.Igual(Vector);
  if abs(Min) < abs(Max) then
  begin
    Vec_aux.PorReal(1 / Max);
    Maximo := Max;
  end
  else
  begin
    Vec_aux.PorReal(1 / Min);
    Maximo := Min;
  end;

  Result := Vec_aux;
end;

function TrasnformarA(MA: TMatR; n: integer): TMatR;
var
  MatP: TMatR;
begin
  MatP := TMatR.Create_Init(MA.nf, MA.nc);
  MatP := Matriz_Potencia_real(MA, 1 / n);

  Result := MatP;

end;


function TrasnformarB(MA, MB: TMatR; n: integer): TMatR;
var
  MatTempo, Kj, Aj, BTemp, Cj, MatP: TMatR;
  VecTemp, Vec1: TvectR;
  i: integer;
  aux: TMatR;
begin
  MatP    := TMatR.Create_Init(MA.nf, MA.nc);
  Cj      := TMatR.Create_Init(MA.nf, MA.nc);
  Aj      := TMatR.Create_Init(MA.nf, MA.nc);
  MatTempo := TMatR.Create_Init(MB.nf, MB.nc);
  BTemp   := TMatR.Create_Init(MA.nf * MB.nf, MA.nc * MB.nc);
  VecTemp := TVectR.Create_Init(MB.nf * MB.nc);
  Vec1    := TVectR.Create_Init(MB.nf * MB.nc);

  //  Kj      := TMatR.Create_Init(MA.nf * MB.nf, MA.nc * MB.nc);
  MatP := Matriz_Potencia_real(MA, 1 / n);
  Kj   := MatP.Kron(MatP);
  Kj.potencia(0);

  Aj.Igual( MatP );
  for i := 1 to n - 1 do
  begin
    aux:= MatP.kron(MatP);
    Kj.sum( aux );
    aux.Free;
    Aj.Mult( Aj, MatP );
  end;
  Kj.inv;
  aux:= MB.Crear_Transpuesta;
  MB.Mult(aux, Cj);
  aux.Free;

  Vec1 := Cj.vec;
  Kj.porVector(Vec1, VecTemp);
  MatTempo := VecTemp.reshape(MB.nf, MB.nc);
  Result   := MatTempo.raiz_Cholesky;
end;


procedure Discretizar(paso_discr: integer; arch: string);
var
  sintetizador_origen, sintetizador_destino: TModeloSintetizadorCEGH;
  arch_salida: string;
  MatA, MatB:  TMatR;
  NSS_origen, NFD_origen, NPFD_origen, NSS_destino, NFD_destino,
  NPFD_destino, kserie, kpaso, kpaso_destino, k, j: integer;
  F_inf, F_sup, alfa: NReal;
  aux_Vect:    TVectR;
  funcionesDeformantesI: TDAOf_ddp_VectDeMuestras;

begin
  Form2.Label1.Caption := arch;
  arch_salida := arch + '111.txt';
  sintetizador_origen := TModeloSintetizadorCEGH.CreateFromArchi(arch);
  sintetizador_destino := TModeloSintetizadorCEGH.CreateFromArchi(arch);
  MatA := TMatR.Create_Init(sintetizador_origen.A_cte.nf, sintetizador_origen.A_cte.nc);
  MatB := TMatR.Create_Init(sintetizador_origen.B_cte.nf, sintetizador_origen.B_cte.nc);
  MatA.Igual(sintetizador_origen.A_cte);
  MatB.Igual(sintetizador_origen.B_cte);
  sintetizador_destino.B_cte.Igual(TrasnformarB(MatA, MatB, paso_discr));
  MatA := sintetizador_origen.A_cte;
  sintetizador_destino.A_cte.Igual(TrasnformarA(MatA, paso_discr));

  // Interpolacion de los datos entre muestras de mayor frecuencia

  NSS_origen  := sintetizador_origen.nBornesSalida;
  NFD_origen  := length(sintetizador_origen.funcionesDeformantes[0]);
  NPFD_origen := sintetizador_origen.funcionesDeformantes[0][0].a.n;

  NSS_destino  := NSS_origen;
  NFD_destino  := NFD_origen * paso_discr;
  NPFD_destino := NPFD_origen;


  for kserie := 0 to NSS_origen - 1 do
  begin
    SetLength(sintetizador_destino.funcionesDeformantes[kserie],
      NFD_destino);
    funcionesDeformantesI :=
      sintetizador_destino.funcionesDeformantes[kserie];
    for kpaso := 1 to NFD_origen do
    begin
      aux_Vect := TVectR.Create_Init(NPFD_origen);
      for k := 1 to paso_discr do
      begin
        alfa := k / paso_discr;
        kpaso_destino := (kpaso - 1) * paso_discr + k - 1;

        for j := 0 to NPFD_origen do

        begin
          F_inf :=
            sintetizador_origen.funcionesDeformantes[kserie][kpaso - 1].a.e(j);
          if kpaso = NFD_origen then
          begin
            F_sup :=
              sintetizador_origen.funcionesDeformantes[kserie][0].a.e(j);
          end
          else
          begin
            F_sup :=
              sintetizador_origen.funcionesDeformantes[kserie][kpaso].a.e(j);
          end;

          aux_Vect.pon_e(
            j, Interpolacion_funciones_deformantes(alfa, F_inf, F_sup));

        end;
        funcionesDeformantesI[kpaso_destino] :=
          Tf_ddp_VectDeMuestras.Create_SinClonarMuestras(aux_Vect, 0);
      end;
    end;

  end;

  sintetizador_destino.WriteToArchi(arch_salida, 0, 2);
  sintetizador_origen.NombresDeBornes_Publicados.Clear;
  sintetizador_destino.NombresDeBornes_Publicados.Clear;
  sintetizador_origen.Free;

end;


function Metodo_potencias(const MatA: TMatR; var VecProp: TMatR): TMatR;
var
  x, y, aux, aux1, VecTemp, Vectemp2: TVectR;
  ii, jj, iter, itermax, ki: integer;

  dist, Norma, landa, V1, vi:  NReal;
  MAtemp, Diagonal, base, Lam: TMatR;

  maux: TMatR;

begin

  x    := TVectR.Create_Init(MatA.nc);
  y    := TVectR.Create_Init(MatA.nc);
  aux  := TVectR.Create_Init(MatA.nc);
  aux1 := TVectR.Create_Init(MatA.nc);
  Vectemp := TVectR.Create_Init(MatA.nc);
  Vectemp2 := TVectR.Create_Init(MatA.nc);

  MAtemp   := TMatR.Create_Init(MatA.nf, MatA.nc);
  Base     := TMatR.Create_Init(MatA.nf, MatA.nc);
  Lam      := TMatR.Create_Init(MatA.nf, MatA.nc);

  Diagonal := Diagonal.Crear_identidad(MatA.nf);
  maux := TMatR.Create_Init(MatA.nf, MatA.nc);


  x.pon_e(1, 1);
  x.pon_e(2,0.1);
  x.pon_e(3,0.1);

  itermax := 100;
  iter    := 1;
  MAtemp  := MatA;
  Norma   := MAtemp.NormMaxAbs;
  begin
    for ii := 1 to MatA.nf do
    begin
      dist := 1;
      while (dist > 0.000001 * Norma) and (iter < itermax) do
      begin
        MAtemp.porVector(x, y);
        y    := Sobre_Max_val_abs(y, landa);
        dist := y.distancia(x);
        aux.Igual(x);
        aux1.Igual(y);
        x.Igual(y);
        iter := iter + 1;
      end;

      Diagonal.pon_e(ii, ii, landa);
      lam.pm[ii].Igual(aux1);
      MAtemp.Igual(MatA);
      jj := ii;

      if y.e(jj) <> 0 then
      begin
        vi := y.e(jj);
        ki := jj;
      end
      else
      begin
        jj := jj + 1;
        if y.e(jj) <> 0 then
        begin
          vi := y.e(jj);
          ki := jj;
        end
        else
        begin
          jj := jj + 1;
          vi := y.e(jj);
          ki := jj;
        end;
      end;

      VecTemp.Igual(MatA.Fila(ki));
      Vectemp.PorReal(1 / (vi * landa));
      Base.pm[ii].Igual(vectemp);
      for jj := 1 to MatA.nf do
      begin
        V1 := -1 * y.e(jj) * landa;
        Vectemp2 := MatA.Fila(jj);
        Vectemp2.sumRPV(V1, Vectemp);
        MAtemp.pm[jj].Igual(Vectemp2);
      end;
    end;
    Result := Diagonal;
  end;


  Vecprop.pm[1].Igual(lam.pm[1]);
  Vecprop.pm[1].HacerUnitario;

  Vecprop.pm[2] := Proyeccion(Diagonal.e(1, 1), Diagonal.e(2, 2), lam.pm[1], lam.pm[2], Base.pm[1]);
  Vecprop.pm[2].HacerUnitario;

  Vecprop.pm[3] := Proyeccion(Diagonal.e(2, 2), Diagonal.e(3, 3), lam.pm[2], lam.pm[3], Base.pm[2]);
  Vecprop.pm[3] := Proyeccion(Diagonal.e(1, 1), Diagonal.e(3, 3), lam.pm[1], Vecprop.pm[3], Base.pm[1]);
  Vecprop.pm[3].HacerUnitario;

  maux:= VecProp.crear_transpuesta;
  VecProp.Igual( maux );
  maux.free;

  x.Free;
  y.Free;
  aux1.Free;
  aux.Free;

end;

procedure muestra(arch: string; var a, b, c: integer);
var
  sintetizador: TModeloSintetizadorCEGH;
begin
  sintetizador := TModeloSintetizadorCEGH.CreateFromArchi(arch);
  a := sintetizador.nBornesSalida;
  b := sintetizador.nPuntosPorPeriodo;
  c := sintetizador.funcionesDeformantes[0][0].a.n;
  sintetizador.Free;

end;

end.

