unit umh_worker_sim;

{$mode delphi}

interface

uses
  xMatDefs, Classes, SysUtils, usalasdejuego, uEstados, umh_sincrodata;

type
  TTareaSimRangoCronicas = class( TTarea_mh )
    kCronicaIni, kCronicaFin: integer;
    constructor Create( nid, kCronicaIni, kCronicaFin: integer);
  end;

  { TRobotOptRangoEstrellas }

  TRobotSimRangoCronicas = class( TWorker_mh )
  private
    procedure copiarVarsTiempoEjecucion;
  public
    sala, salaMadre: TSalaDeJuego;
    nTareasCalculadasEstePaso: integer;
  public
    constructor Create(
      nid: integer;
      sincroData: TSincroData_mh;
      const sala: TSalaDeJuego );
    procedure Execute; override;
  end;


implementation



constructor TTareaSimRangoCronicas.Create( nid, kCronicaIni, kCronicaFin: integer );
begin
  inherited Create( nid );
  self.kCronicaIni:= kCronicaIni;
  self.kCronicaFin:= kCronicaFin;
end;

constructor TRobotSimRangoCronicas.Create(
      nid: integer;
      sincroData: TSincroData_mh;
      const sala: TSalaDeJuego );
begin
  inherited Create( nid, sincroData );
  self.sala := TSalaDeJuego.cargarSala( sala.archiSala, sala.EscenarioActivo.nombre, true);
  salaMadre := sala;
  self.sala.idHilo := nid + 1;
  copiarVarsTiempoEjecucion;
  sala.estabilizarInicio := False;
  nTareasCalculadasEstePaso := 0;
end;



procedure TRobotSimRangoCronicas.copiarVarsTiempoEjecucion;
begin
{$IFDEF ITERAR}
  sala.globs.NMAX_ITERACIONESDELPASO_OPT :=
    salaMadre.globs.NMAX_ITERACIONESDELPASO_OPT;
  sala.globs.NMAX_ITERACIONESDELPASO_SIM :=
    salaMadre.globs.NMAX_ITERACIONESDELPASO_SIM;
{$ENDIF}
  sala.globs.SortearOpt := SalaMadre.globs.SortearOpt;
  sala.escribirOptActores := SalaMadre.escribirOptActores;
  sala.globs.TasaDeActualizacion := SalaMadre.globs.TasaDeActualizacion;
  sala.globs.NCronicasOpt := SalaMadre.globs.NCronicasOpt;
  sala.globs.NCronicasSim := 1;
  sala.globs.semilla_inicial_opt := SalaMadre.globs.semilla_inicial_opt;
  sala.globs.semilla_inicial_sim := SalaMadre.globs.semilla_inicial_sim;
end;




procedure TRobotSimRangoCronicas.Execute;
var
  pTarea: TTareaSimRangoCronicas;
begin
  nTareasCalculadasEstePaso := 0;
  pTarea:= sincrodata_mh.wrk_AsignarTarea( self, nil ) as TTareaSimRangoCronicas;
  aTarea:= pTarea;
  if pTarea = nil then
  begin
   Terminate;
  end;

  while ( sincrodata_mh.Paso > 0) and (not Terminated) do
  begin
    {
    writeln( 'Robot: '+IntToStr( nid )+' kPaso: '+IntToStr( sincrodata_mh.Paso )+
    ', kEstrellaINI: '+IntTostr( pTarea.estrellaIni ) +', kEstrellaFin: '+IntTostr( pTarea.estrellaFin ));
     }

    sala.SimularRangoCronicas( pTarea.kCronicaIni, pTarea.kCronicaFin, true );
//    writeln( 'Robot: '+IntToStr( nid )+' Calcular_fin '+DateTimeTostr( now ) );
    inc(nTareasCalculadasEstePaso);
    pTarea:= sincrodata_mh.wrk_AsignarTarea( self, aTarea ) as TTareaSimRangoCronicas;
    aTarea:= pTarea;
    if aTarea = nil then
    begin
     Terminate;
    end;
  end; //while de los pasos

  sala.Free;

  // aquí podría ir
  sincrodata_mh.wrk_Finalizado( Self );
end;

end.
