unit winLinuxUtils;

interface

uses
{$IFDEF WIN32}
  Windows,
{$ELSE}
  ctypes,
{$ENDIF}
  Sysutils,
  Classes;

{$IFDEF LINUX}
const
  barraDir= '/';
{$ELSE}
const
  barraDir= '\';
{$ENDIF}

//Retorna todos los archivos en path cuya extension se encuentre en extensiones
//y sus subdirectorios
//Las extensiones deben darse sin ., un archivo .exe se buscara con exe
//Si se quiere cualquier extension poner '*' en extensiones
procedure getDirectoryContents(path: String; extensiones: array of String; var res: TStringList; recursivo: boolean); overload;

// OJO!. Elimina todos los archivos en la carpeta "path" que pasen por el filtro de extensiones..
procedure deleteFiles(path: String; extensiones: array of String;  recursivo: boolean);


function GetSystemCoreCount: integer;

//Si esta en windows cambia las '/' por '\'
//Si esta en linux cambia las '\' por '/'
procedure toOSBarraDirs(var s: String);

implementation




procedure getDirectoryContents(path: String; extensiones: array of String; var res: TStringList; recursivo: boolean);
var
  sr: TSearchRec;
  FileAttrs: Integer;
  directorios: TStringList;
  pathExtension: String;
  i: Integer;
begin
{  FileAttrs:= 0;
  FileAttrs := FileAttrs + faReadOnly;
  FileAttrs := FileAttrs + faHidden;
  FileAttrs := FileAttrs + faSysFile;
  FileAttrs := FileAttrs + faVolumeID;
  FileAttrs := FileAttrs + faDirectory;
  FileAttrs := FileAttrs + faArchive;
  FileAttrs := FileAttrs + faAnyFile;}

  if path[Length(path)] <> barradir then
    path:= path + barraDir;

  FileAttrs := 0;
  for i:= 0 to high(extensiones) do
  begin
    pathExtension:= path + '*.' + extensiones[i];
    if SysUtils.FindFirst(pathExtension, FileAttrs, sr) = 0 then
    begin
      repeat
        if (sr.Attr and FileAttrs) = FileAttrs then
          res.Add(path + sr.Name);
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;
  end;

  if recursivo then
  begin
  directorios:= TStringList.Create;
  FileAttrs:= faDirectory;
  pathExtension:= path + '*.*';
  if SysUtils.FindFirst(pathExtension, FileAttrs, sr) = 0 then
  begin
    repeat
      if ((sr.Attr and FileAttrs) = FileAttrs) and
         (sr.Name <> '.') and   //Solo busco en directorios hacia abajo
         (sr.Name <> '..') then
        directorios.Add(sr.Name);
    until FindNext(sr) <> 0;
    FindClose(sr);
  end;

  for i:= 0 to directorios.Count - 1 do
      getDirectoryContents(path + directorios[i] + barraDir, extensiones, res, recursivo);
  directorios.Free;
end;
end;

procedure deleteFiles(path: String; extensiones: array of String; recursivo: boolean);
var
  archis: TStringList;
  i: Integer;
begin
  getDirectoryContents(path, extensiones, archis, recursivo);
  for i:= 0 to archis.Count - 1 do
    DeleteFile(archis[i]);
  archis.Free;
end;

{$IFDEF Linux}
const _SC_NPROCESSORS_ONLN = 83;
function sysconf(i: cint): clong; cdecl; external name 'sysconf';
{$ENDIF}
//Parte de la funcin encontrada en
//http://www.nabble.com/number-of-cpu-cores-td20747886.html
//La funcion original era GetSystemThreadCount
function GetSystemCoreCount: integer;
// returns a good default for the number of threads on this system
{$IFDEF WIN32}
//returns total number of processors available to system including logical hyperthreaded processors
var
  i: Integer;
  ProcessAffinityMask, SystemAffinityMask: DWORD;
  Mask: DWORD;
  SystemInfo: SYSTEM_INFO;
  res: integer;
begin
  if GetProcessAffinityMask(GetCurrentProcess, ProcessAffinityMask, SystemAffinityMask) then
  begin
    res := 0;
    Mask:= 1;
    for i := 0 to 31 do
    begin
      if (ProcessAffinityMask and Mask) <> 0 then
        inc( res );
      Mask:= Mask shl 1;
    end;
  end
  else
  begin
    //can't get the affinity mask so we just report the total number of processors
    GetSystemInfo(SystemInfo);
    res:= SystemInfo.dwNumberOfProcessors;
  end;
  result:= res;
end;
{$ELSE}
{$IFDEF LINUX}
begin
//  Result:= sysconf(_SC_NPROCESSORS_ONLN);  ME FALLA EN EL CLUSTER
   result:= 1; //???? no se como.
(* ???
rchrch@20090910: no me funciono la llamada a sysconf en el cluster.
palfaro@20090915: Mirate ac: http://www.brandonhutchinson.com/Understanding_proc_cpuinfo.html
  aparentemente hay un archivo con info del sistema que tiene la cantidad de procesadores
rch@20090916: cuando tengamos ms tiempo lo vemos en el cluster.
  por ahora no importa pues estoy largando cada proceso
  por separado sin saber si los nucleos estn en la misma
  maquina y entonces la respuesta 1 esta ok.
  igual dejamos estos comentarios en el cdigo para acordarnos.
*)
end;
{$ELSE}
begin
  Result:= 1;
end;
{$ENDIF}
{$ENDIF}

procedure toOSBarraDirs(var s: String);
begin
{$IFDEF WIN32}
  s:= StringReplace(s, '/', barraDir, [rfReplaceAll]);
{$ELSE}
  s:= StringReplace(s, '\', barraDir, [rfReplaceAll]);
{$ENDIF}
end;

end.
